package djmil.cordacheckers.api;

import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.Rank;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.RspFailureException;
import djmil.cordacheckers.user.HoldingIdentityResolver;


@RestController
@RequestMapping("api/leaderboard")
public class LeaderboardController {

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @Autowired
    CordaClient cordaClient;

    @GetMapping
    public ResponseEntity<Map<String, Rank>> getLeaderboard() {
        try {
            final var hiCustodian  = holdingIdentityResolver.getCustodian();
            final Map<String, Rank> leaderboard = cordaClient.fetchRanking(hiCustodian);

            return ResponseEntity.ok(leaderboard);
        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

}
