package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import org.jetbrains.annotations.Nullable;

import djmil.cordacheckers.contracts.GameResultContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;

@BelongsToContract(GameResultContract.class)
public class GameResultState extends GameState {

    private final MemberX500Name winnerName; // NULL if it is a draw
    private final Integer totalMoves;
 
    @ConstructorForDeserialization
    public GameResultState(MemberX500Name whitePlayer, MemberX500Name blackPlayer, MemberX500Name winnerName, 
            Integer totalMoves, UUID gameUuid, String message, List<PublicKey> participants) {
        super(whitePlayer, blackPlayer, gameUuid, message, participants);
        this.winnerName = winnerName;
        this.totalMoves = totalMoves;
    }

    public GameResultState(MemberX500Name winnerName, GameBoardState gameBoardState) {
        super(gameBoardState.whitePlayer, gameBoardState.blackPlayer, gameBoardState.gameUuid, null, gameBoardState.participants);
        this.winnerName = winnerName;
        this.totalMoves = gameBoardState.getMoveNumber();
    }

    @Nullable
    public MemberX500Name getWinnerName() {
        return winnerName;
    }

    public MemberX500Name getLooserName() {
        return getOpponentName(getWinnerName());
    }

    public Integer getTotalMoves() {
        return totalMoves;
    }

}
