import React from 'react';
import { Color } from '../../components/Checkers';
import Wobler from '../../components/Wobler';
import './ActionPanel.css'

/*
 * NewGame actoins
 */
export function Create({ getGame, onClick }) {
    const [game, isPushing] = getGame();
    const hasPlayers = (game.opponentName && game.myColor) ? true : '';

    const validate = () => {
        if (!hasPlayers)
            return alert("You have to select an opponent");

        if (isPushing)
            return; // busy

        onClick({
            opponentName: game.opponentName,
            opponentColor: Color.opposite(game.myColor), // TODO: by fixing this i can simply return GAME
            board: game.board,
            message: game.message
        });
    }

    return (
        <button className={'Create' + (hasPlayers && ' ready')} onClick={() => validate()} >
            <Wobler text="Create" dance={isPushing} />
        </button>
    )
}

/*
 * GameProposal actions
 */
export function Accept({ getGame, onClick }) {
    const [game, isPushing] = getGame();
    const hasGame = (game.status === 'GAME_PROPOSAL_WAIT_FOR_YOU') ? true : '';

    const validate = () => {
        if (!hasGame)
            return alert('You have to select pending GameProposal');

        if (isPushing)
            return; // busy

        onClick({ uuid: game.uuid });
    }

    if (game.status !== 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        return (
            <button className={'Accept' + (hasGame && ' ready')} onClick={() => validate()}>
                <Wobler text="Accept" dance={isPushing === 'accept'} />
            </button>
        )
}

export function Reject({ getGame, onClick }) {
    const [game, isPushing] = getGame();
    const hasGame = (game.status === 'GAME_PROPOSAL_WAIT_FOR_YOU') ? true : '';

    const validate = () => {
        if (!hasGame)
            return alert('You have to select some GameProposal');

        if (isPushing)
            return; // busy

        onClick({ uuid: game.uuid });
    }

    if (game.status !== 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        return (
            <button className={'Reject' + (hasGame && ' ready')} onClick={() => validate()} >
                <Wobler text="Reject" dance={isPushing === 'reject'} />
            </button>
        )
}

export function Cancel({ getGame, onClick }) {
    const [game, isPushing] = getGame();

    if (game.status !== 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        return;

    const hasGame = (game.status === 'GAME_PROPOSAL_WAIT_FOR_OPPONENT') ? true : '';

    const validate = () => {
        if (!hasGame)
            return alert('You have to select pending GameProposal');

        if (isPushing)
            return; // busy

        onClick({ uuid: game.uuid });
    }

    return (
        <button className={'Cancel' + (hasGame && ' ready')} onClick={() => validate()} >
            <Wobler text="Cancel" dance={isPushing === 'cancel'} />
        </button>
    )
}

/*
 * Game actions
 */
export function Surrender({ getGame, onClick }) {
    const [game, isPushing] = getGame();

    if (game.status === 'DRAW_REQUEST_WAIT_FOR_YOU' || game.status === 'DRAW_REQUEST_WAIT_FOR_OPPONENT')
        return; // You shall not surrender if there is an active tie negotiations

    const hasGame = (game.status === 'GAME_BOARD_WAIT_FOR_OPPONENT' || game.status === 'GAME_BOARD_WAIT_FOR_YOU') ? true : '';

    const validate = () => {
        if (!hasGame)
            return alert('You have to select a Game');

        if (isPushing)
            return; // busy

        onClick({ uuid: game.uuid });
    }

    return (
        <button className={'Surrender' + (hasGame && ' ready')} onClick={() => validate()} >
            <Wobler text="Surrender" dance={isPushing === 'surrender'} />
        </ button>
    )
}

/*
 * Game actions: Draw
 */
export function DrawRequest({ getGame, onClick }) {
    const [game, isPushing] = getGame();

    if (game.status === 'DRAW_REQUEST_WAIT_FOR_YOU')
        return; // You can not send counter draw request

    const hasGame = (game.status === 'GAME_BOARD_WAIT_FOR_YOU') ? true : '';

    const validate = () => {
        if (!hasGame)
            return alert('You can ask for a draw only during your turn in a Game');

        if (game.status === 'DRAW_REQUEST_WAIT_FOR_OPPONENT')
            return alert('A draw was alredy offered to the opponent');

        onClick({ uuid: game.uuid });
    }


    return (
        <button className={'Draw' + (hasGame && ' ready')} onClick={() => validate()} >
            <Wobler text="Draw?" dance={isPushing === 'draw_request'} />
        </button >
    )
}

export function DrawAccept({ getGame, onClick }) {
    const [game, isPushing] = getGame();

    if (game.status !== 'DRAW_REQUEST_WAIT_FOR_YOU')
        return;

    return (
        <button className='Draw accept' onClick={() => onClick({ uuid: game.uuid })} >
            <Wobler text="Draw accept" dance={isPushing === 'draw_accept'} />
        </button>
    )
}

export function DrawReject({ getGame, onClick }) {
    const [game, isPushing] = getGame();

    if (game.status !== 'DRAW_REQUEST_WAIT_FOR_YOU')
        return;

    return (
        <button className='Draw reject' onClick={() => onClick({ uuid: game.uuid })}>
            <Wobler text="Reject" dance={isPushing === 'draw_reject'} />
        </button>
    )
}

/*
 * GameArchive actions
 */
export function Backward() {
    return <button className='Backward' disabled>Backward</button>
}

export function Forward() {
    return <button className='Forward' disabled>Forward</button>
}