import React from 'react';
import { Color, Player, Board } from '../../components/Checkers';
import './GameBoard.css';
import BadMove from './BadMove';

export default function GameBoard({ dispatchGuide, username, getGame, onStoneClick, onStoneMove }) {
  const [game, isPushing, error] = getGame();

  const myName = game.opponentName ? username : '';
  const opponentColor = Color.opposite(game.myColor);
  const flipBoard = (game.myColor === Color.black) ? true : null;

  const optionalOnStoneClick = (typeof onStoneClick !== 'function') ? null :
    (cellId) => onStoneClick(game.uuid, cellId);

  const optionalOnStoneMove = (typeof onStoneMove !== 'function' || isPushing) ? null :
    (move) => {
      if (move[0] === move[1] || game.board[move[1]] !== undefined)
        return; // ignore obviosly wrong move

      if (game.status === 'DRAW_REQUEST_WAIT_FOR_YOU')
        return alert("You have to descide on Draw request from " + game.opponentName);

      if (game.status === 'DRAW_REQUEST_WAIT_FOR_OPPONENT')
        return alert("Awating for " + game.opponentName + "'s decision on the Draw request");

      onStoneMove(game.uuid, move)
    };

  const currMove = (() => {
    if (isPushing) return [isPushing.moveFrom, isPushing.moveTo];
    if (error?.moveFrom && error?.moveTo) return [error.moveFrom, error.moveTo];
    return [];
  })(); // <<-- execute

  return (
    <div className='GameBoard'>
      <Player
        color={opponentColor || Color.black}
        name={game.opponentName}
      />
      <Board
        board={game.board}
        flip={flipBoard}
        prevMove={game.previousMove}
        currMove={currMove}
        onStoneClick={optionalOnStoneClick}
        onStoneMove={optionalOnStoneMove}
      />
      <Player
        color={game.myColor || Color.white}
        name={myName}
      />
      <BadMove dispatchGuide={dispatchGuide} uuid={game?.uuid} message={error?.message} />
    </div>
  )
}